/*============================================================================
  File:     IndexUsageStats.sql

  Summary:  Be careful... don't just drop all unused indexes! Please use
			this script with caution. If your persisted information is not
			comprehensive then you might be dropping useful indexes - just
			not indexes used since you started persisting the data!!!
           
  Date:     April 2010

  SQL Server Version: 2005/2008
------------------------------------------------------------------------------
  Written by Kimberly L. Tripp, SYSolutions, Inc.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Generically look at all indexes - and persist this data so that you can analyze over time

CREATE DATABASE [PerformanceDW]
go 

USE [PerformanceDW]
go

-- drop the table... 
-- drop table [persisted-dm_db_index_usage_stats]

-- Create the permanent table
SELECT getdate() AS RunTime 
	, DB_NAME(i.database_id) as DatabaseName
	, OBJECT_NAME(i.object_id, i.database_id) as ObjectName
	, *
	, 1 AS [Rank]
INTO [persisted-dm_db_index_usage_stats]
FROM sys.dm_db_index_usage_stats AS i
WHERE 1=2 
go

INSERT [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats]
SELECT getdate()  
	, DB_NAME(i.database_id) 
	, OBJECT_NAME(i.object_id, i.database_id) 
	, *
FROM 
	(SELECT *, 
	DENSE_RANK() OVER (PARTITION BY database_id ORDER BY database_id ASC) AS [Rank]
	FROM sys.dm_db_index_usage_stats) AS i
WHERE object_id > 100 AND DB_NAME(i.database_id) NOT IN ('master', 'model', 'msdb', 'tempdb')
go

SELECT *
FROM [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats]
WHERE object_id > 100
ORDER BY DatabaseName ASC
go

-- Verify the indexes NOT used for a particular database
-- Remember, these are the indexes NOT used since you started gathering "usage stats"
-- If you have NOT gathered stats over a true "business cycle" then you may NOT
-- have accurate stats (for example for month end processing) and therefore you 
-- might actually drop an index that's very critical for a report or other and
-- that might have a profound affect on overall performance (during month-end)...

USE CREDIT
go

SELECT object_name(si.object_id) AS 'TableName'
	, si.index_id
	, si.name AS 'IndexName'
	, N'DROP INDEX ' 
		+ QUOTENAME(object_name(si.object_id), N']') 
		+ N'.' 
		+ QUOTENAME(si.name, N']') AS 'Drop Index Statement'
FROM sys.indexes AS si 
WHERE 
	( -- this first statement looks for indexes that aren't listed AT ALL in the usage stats
		si.index_id NOT IN (SELECT pStats.index_id 
            FROM [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats] AS pStats 
            WHERE pStats.object_id = si.object_id AND 
                  si.index_id = pStats.index_id AND 
                  database_id = db_id()) 
    OR -- this second statement looks for indexes that aren't being used for seeks/scans but have updates
		si.index_id IN (SELECT pStats.index_id 
            FROM [PerformanceDW].dbo.[persisted-dm_db_index_usage_stats] AS pStats 
            WHERE pStats.object_id = si.object_id AND 
                  si.index_id = pStats.index_id AND 
                  database_id = db_id()
                  AND pStats.user_seeks < 0			-- may want to adjust this (possibly, start with 5)
                  AND pStats.user_scans < 0			-- may want to adjust this (possibly, start with 5)
                  AND pStats.user_updates > 1)		-- may want to adjust this (possibly, start with 100)
   	)
AND si.object_id > 100 
	AND si.index_id > 0 -- don't drop heaps (it would fail regardless because you can't drop a heap with DROP INDEX...)
	AND si.is_primary_key = 0
	AND si.is_unique_constraint = 0
	AND (object_name(si.object_id) NOT LIKE 'queue_messages%'	-- these are "system" table indexes for tables that are NOT marked as system tables
        AND object_name(si.object_id) NOT LIKE 'fulltext%'	
        AND object_name(si.object_id) NOT LIKE 'ifts%'
        AND object_name(si.object_id) NOT LIKE 'filestream%'
        AND object_name(si.object_id) NOT LIKE 'syscommittab%')
ORDER BY object_name(si.object_id), si.index_id
go

-----------------------------------------------------------------------
-- Here's a copy of the DMV query if you just want to run it once
-- but include the database and object name instead of just IDs:
-----------------------------------------------------------------------

SELECT getdate() AS RunTime 
	, DB_NAME(i.database_id) as DatabaseName
	, OBJECT_NAME(i.object_id, i.database_id) as ObjectName
	, *
FROM sys.dm_db_index_usage_stats AS i
WHERE object_id > 100
		AND -- although this is debatable as you *might* want to add indexes to these non-system, system tables. ;-)
		(	OBJECT_NAME(i.object_id, i.database_id) NOT LIKE 'queue_messages%'	-- these are "system" table indexes for tables that are NOT marked as system tables
			AND OBJECT_NAME(i.object_id, i.database_id) NOT LIKE 'fulltext%'	
			AND OBJECT_NAME(i.object_id, i.database_id) NOT LIKE 'ifts%'
			AND OBJECT_NAME(i.object_id, i.database_id) NOT LIKE 'filestream%'
			AND OBJECT_NAME(i.object_id, i.database_id) NOT LIKE 'syscommittab%'
		)
ORDER BY DatabaseName, ObjectName, i.index_id
go